@@ -3,7 +3,7 @@
     Copyright 2008-2011  Tomaz Canabrava <tomaz.canabrava@gmail.com>
     Copyright 2008       Ugo Sangiori <ugorox@gmail.com>
     Copyright 2010-2011  Wagner Reck <wagner.reck@gmail.com>
-    Copyright 2011-2012  Andreas Cord-Landwehr <cola@uni-paderborn.de>
+    Copyright 2011-2013  Andreas Cord-Landwehr <cola@uni-paderborn.de>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
@@ -27,11 +27,9 @@
 #include "PointerType.h"
 #include "DataType.h"
 #include "DataStructureBackendManager.h"
-#include "DataStructurePluginInterface.h"
+#include "DataStructureBackendInterface.h"
 #include "QtScriptBackend.h"
-
-#include <boost/shared_ptr.hpp>
-#include <boost/concept_check.hpp>
+#include "ConcurrentHelpClasses.h"
 
 #include <KStandardDirs>
 #include <KSaveFile>
@@ -45,9 +43,12 @@
 class DocumentPrivate
 {
 public:
-    DocumentPrivate() {}
-
-    enum borderSize { SceneBorder = 50 };
+    DocumentPrivate()
+        : _left(0)
+        , _right(0)
+        , _top(0)
+        , _bottom(0)
+    { }
 
     QString _lastSavedDocumentPath;
     QString _name;
@@ -55,12 +56,9 @@ public:
     qreal _right;
     qreal _top;
     qreal _bottom;
-    qreal _minWidth;
-    qreal _minHeight;
     bool _modified;
-    bool _saved;
     DataStructurePtr _activeDataStructure;
-    QPointer<DataStructurePluginInterface> _backend;
+    QPointer<DataStructureBackendInterface> _backend;
     QtScriptBackend* _engineBackend;
     QList< DataStructurePtr > _dataStructures;
 
@@ -74,19 +72,13 @@ public:
 
 
 
-Document::Document(const QString& name, qreal left, qreal right, qreal top, qreal bottom, QObject *parent)
-    : QObject(parent), d(new DocumentPrivate())
+Document::Document(const QString& name, QObject *parent)
+    : QObject(parent)
+    , d(new DocumentPrivate())
 {
     d->_name = name;
-    d->_left = left;
-    d->_right = right;
-    d->_top = top;
-    d->_bottom = bottom;
-    d->_minWidth = 0;
-    d->_minHeight = 0;
-    d->_saved = false;
     d->_engineBackend = new QtScriptBackend(this);
-    d->_backend = DataStructureBackendManager::self()->activeBackend();
+    d->_backend = DataStructureBackendManager::self().activeBackend();
     d->_modified = false;
 
     d->_iconPackage = KGlobal::dirs()->locate("appdata", "iconpacks/default.svg");
@@ -109,14 +101,15 @@ void Document::setModified(const bool mod)
 void Document::changeBackend()
 {
     cleanUpBeforeConvert();
-    d->_backend = DataStructureBackendManager::self()->activeBackend();
+    d->_backend = DataStructureBackendManager::self().activeBackend();
 
     // create list of existing data structures, then convert them one by one
     QList<DataStructurePtr> dataStructures = QList<DataStructurePtr>(d->_dataStructures);
     d->_dataStructures.clear();
 
     for (int i = 0; i < dataStructures.count(); ++i) {
-        DataStructurePtr newDataStructure = addDataStructure(DataStructureBackendManager::self()->createDataStructure(dataStructures.at(i), this));
+        DataStructurePtr newDataStructure = addDataStructure(
+                DataStructureBackendManager::self().createDataStructure(dataStructures.at(i), this));
         // remove origin data structure
         dataStructures[i]->remove();
     }
@@ -262,195 +255,49 @@ QString Document::name() const
     return d->_name;
 }
 
-// gets the wheight of the drawable area
-qreal Document::height() const
+void Document::updateGraphics(DataTypePtr dataType)
 {
-    return d->_bottom - d->_top;
+    foreach(const DataStructurePtr &structure, d->_dataStructures) {
+        QtConcurrent::blockingMap(structure->dataList(dataType->identifier()), DataGraphicsUpdated());
+    }
 }
 
-// sets the width of the drawable area
-qreal Document::width() const
+void Document::updateGraphics(PointerTypePtr pointerType)
 {
-    return d->_right - d->_left;
+    foreach(const DataStructurePtr &structure, d->_dataStructures) {
+        QtConcurrent::blockingMap(structure->pointers(pointerType->identifier()), PointerGraphicsUpdated());
+    }
 }
 
-qreal Document::left() const
-{
-    return d->_left;
-}
-
-qreal Document::right() const
-{
-    return d->_right;
-}
-
-qreal Document::top() const
-{
-    return d->_top;
-}
-
-qreal Document::bottom() const
-{
-    return d->_bottom;
-}
-
-void Document::setLeft(qreal leftValue)
-{
-    d->_left = leftValue;
-    //d->_modified = true;
-}
-
-void Document::setRight(qreal rightValue)
-{
-    d->_right = rightValue;
-    //d->_modified = true;
-}
-
-void Document::setTop(qreal topValue)
-{
-    d->_top = topValue;
-    //d->_modified = true;
-}
-
-void Document::setBottom(qreal bottomValue)
-{
-    d->_bottom = bottomValue;
-    //d->_modified = true;
-}
-
-QRectF Document::size()
+QRectF Document::sceneRect() const
 {
     return QRectF(d->_left, d->_top, d->_right - d->_left, d->_bottom - d->_top);
 }
 
-bool Document::isPointAtDocument(qreal x, qreal y)  const
+void Document::updateSceneRect(const QPointF &position)
 {
-    if (x < d->_left)      return false;
-    if (x > d->_right)     return false;
-    if (y < d->_top)       return false;
-    if (y > d->_bottom)    return false;
-
-    return true;
-}
-
-void Document::changeMinimalSize(qreal width, qreal height)
-{
-    if (width >= 0) d->_minWidth = width;
-    if (height >= 0) d->_minHeight = height;
-
-    if (width > d->_right - d->_left) {
-        d->_left -= (d->_right - d->_left - width) / 2;
-        d->_right += (d->_right - d->_left - width) / 2;
-        emit resized();
-    } else {
-        resizeDocumentBorder(BorderLeft);
+    if (sceneRect().contains(position) == true) {
+        //TODO If this happens, we must shrink the rectangular area where the nodes
+        // are positioned. Though it looks that one of the only efficient ways to do this
+        // is to implement a QuadTree for the scene rect an track the positions of nodes.
+        // Especially for corner cases (moving a node at the border of the rect with epsilon small
+        // wobbling) we must take a carefull look to guarantee fast processing.
+        return;
+    }
+    if (position.x() < d->_left) {
+        d->_left = position.x();
+    }
+    if (position.x() > d->_right) {
+        d->_right = position.x();
+    }
+    if (position.y() < d->_top) {
+        d->_top = position.y();
+    }
+    if (position.y() > d->_bottom) {
+        d->_bottom = position.y();
     }
 
-    if (height > d->_bottom - d->_top) {
-        d->_top -= (d->_bottom - d->_top - height) / 2;
-        d->_bottom += (d->_bottom - d->_top - height) / 2;
-        emit resized();
-    } else {
-        resizeDocumentBorder(BorderTop);
-    }
-}
-
-void Document::resizeDocumentIncrease()
-{
-    int elements = dataStructures().size();
-    for (int i = 0; i < elements; i++) {
-        bool resizeDocument = false;
-        foreach(DataPtr n,  dataStructures().at(i)->dataList()) {
-            if (n->x() < d->_left + DocumentPrivate::SceneBorder) {
-                setLeft(d->_left - DocumentPrivate::SceneBorder);
-                resizeDocument = true;
-            }
-            if (n->x() > d->_right - DocumentPrivate::SceneBorder) {
-                setRight(d->_right + DocumentPrivate::SceneBorder);
-                resizeDocument = true;
-            }
-            if (n->y() < d->_top + DocumentPrivate::SceneBorder) {
-                setTop(d->_top - DocumentPrivate::SceneBorder);
-                resizeDocument = true;
-            }
-            if (n->y() > d->_bottom - DocumentPrivate::SceneBorder) {
-                setBottom(d->_bottom + DocumentPrivate::SceneBorder);
-                resizeDocument = true;
-            }
-        }
-        if (resizeDocument) {
-            emit resized();
-        }
-    }
-}
-
-void Document::resizeDocumentBorder(Document::Border orientation)
-{
-    bool empty = true;
-    int elements = dataStructures().size();
-
-    // scans doubled border of specified size: if empty or not
-    for (int i = 0; i < elements; i++) {
-        foreach(DataPtr n,  dataStructures().at(i)->dataList()) {
-            switch (orientation) {
-            case BorderLeft: {
-                if (n != 0 && n->x() < d->_left + DocumentPrivate::SceneBorder * 2) empty = false;
-                break;
-            }
-            case BorderRight: {
-                if (n != 0 && n->x() > d->_right - DocumentPrivate::SceneBorder * 2) empty = false;
-                break;
-            }
-            case BorderTop: {
-                if (n != 0 && n->y() < d->_top + DocumentPrivate::SceneBorder * 2) empty = false;
-                break;
-            }
-            case BorderBottom: {
-                if (n != 0 && n->y() > d->_bottom - DocumentPrivate::SceneBorder * 2) empty = false;
-                break;
-            }
-            }
-        }
-    }
-
-    //TODO connect to graphvisualeditor-size
-    if (empty == true) {
-        switch (orientation) {
-        case BorderLeft: {
-            if (d->_right - d->_left < DocumentPrivate::SceneBorder * 4)
-                return;
-            setLeft(d->_left + DocumentPrivate::SceneBorder);
-            emit resized();
-            break;
-        }
-        case BorderRight: {
-            if (d->_right - d->_left < DocumentPrivate::SceneBorder * 4)
-                return;
-            setRight(d->_right - DocumentPrivate::SceneBorder);
-            emit resized();
-            break;
-        }
-        case BorderTop: {
-            if (d->_bottom - d->_top < DocumentPrivate::SceneBorder * 4)
-                return;
-            setTop(d->_top + DocumentPrivate::SceneBorder);
-            emit resized();
-            break;
-        }
-        case BorderBottom: {
-            if (d->_bottom - d->_top < DocumentPrivate::SceneBorder * 4)
-                return;
-            setBottom(d->_bottom - DocumentPrivate::SceneBorder);
-            emit resized();
-            break;
-        }
-        }
-    }
-}
-
-bool Document::isPointAtDocument(const QPointF& point)  const
-{
-    return isPointAtDocument(point.x(), point.y());
+    emit sceneRectChanged(sceneRect());
 }
 
 bool Document::isModified() const
@@ -485,8 +332,9 @@ void Document::setActiveDataStructure(DataStructurePtr g)
 
 DataStructurePtr Document::addDataStructure(const QString& name)
 {
-    DataStructurePtr dataStructure = DataStructureBackendManager::self()->createDataStructure(this,
-                         d->_backend->internalName());
+kDebug() << "add structure";
+    DataStructurePtr dataStructure = DataStructureBackendManager::self().createDataStructure(
+            this, d->_backend->internalName());
     dataStructure->setName(name);
     return addDataStructure(dataStructure);
 }
@@ -514,9 +362,13 @@ DataStructurePtr Document::addDataStructure(DataStructurePtr dataStructure)
     d->_dataStructures.append(dataStructure);
     d->_activeDataStructure = dataStructure;
     d->_modified = true;
+
     connect(dataStructure.get(), SIGNAL(changed()), this, SLOT(setModified()));
+    connect(dataStructure.get(), SIGNAL(dataPositionChanged(QPointF)), this, SLOT(updateSceneRect(QPointF)));
+
     emit dataStructureCreated(dataStructure);
     emit dataStructureListChanged();
+
     return dataStructure;
 }
 
@@ -544,35 +396,36 @@ void Document::setFileUrl(const KUrl& fileUrl)
 void Document::remove(DataStructurePtr dataStructure)
 {
     d->_dataStructures.removeOne(dataStructure);
-    if (d->_dataStructures.count() == 0) {
-        addDataStructure();
+    if (d->_dataStructures.count() != 0) {
+        setActiveDataStructure(d->_dataStructures.count() - 1);
+    } else {
+        d->_activeDataStructure.reset();
     }
-    setActiveDataStructure(d->_dataStructures.count()-1);
     d->_modified = true;
     emit dataStructureListChanged();
 }
 
 void Document::clear()
 {
+    // remove data structures
+    d->_activeDataStructure.reset();
+    foreach (const DataStructurePtr &dataStructure, d->_dataStructures) {
+        dataStructure->remove();
+    }
+    d->_dataStructures.clear();
+
     // remove types
-    for (QMap<int,DataTypePtr>::const_iterator iter= d->_dataTypes.begin(); iter != d->_dataTypes.end(); ++iter) {
+    for (QMap<int,DataTypePtr>::const_iterator iter= d->_dataTypes.constBegin(); iter != d->_dataTypes.constEnd(); ++iter) {
         emit(dataTypeRemoved(iter.key()));
         d->_dataTypes[iter.key()]->remove();
     }
     d->_dataTypes.clear();
 
-    for (QMap<int,PointerTypePtr>::const_iterator iter= d->_pointerTypes.begin(); iter != d->_pointerTypes.end(); ++iter) {
+    for (QMap<int,PointerTypePtr>::const_iterator iter= d->_pointerTypes.constBegin(); iter != d->_pointerTypes.constEnd(); ++iter) {
         emit(pointerTypeRemoved(iter.key()));
         d->_pointerTypes[iter.key()]->remove();
     }
     d->_pointerTypes.clear();
-
-    // remove data structures
-    foreach (const DataStructurePtr &dataStructure, d->_dataStructures) {
-        dataStructure->remove();
-    }
-    d->_activeDataStructure.reset();
-    d->_dataStructures.clear();
 }
 
 DataStructurePtr Document::activeDataStructure() const
@@ -580,14 +433,14 @@ DataStructurePtr Document::activeDataStructure() const
     return d->_activeDataStructure;
 }
 
-DataStructurePluginInterface * Document::backend() const
+DataStructureBackendInterface * Document::backend() const
 {
     return d->_backend;
 }
 
 void Document::setBackend(const QString &pluginIdentifier)
 {
-    DataStructurePluginInterface * plugin = DataStructureBackendManager::self()->backend(pluginIdentifier);
+    DataStructureBackendInterface * plugin = DataStructureBackendManager::self().backend(pluginIdentifier);
     if (plugin) {
         d->_backend = plugin;
     }
